import sys
import time
import os
import urllib

import requests


def get_keycloak_realm_and_client(mlflow_url: str):
    """
    NCSA Deployments of MLFlow are protected by the OAuth2-proxy sidecar. The OAuth2
    start endpoint redirects to our keycloak instance. Find this redirect and morph the
    redirect URL into a device flow start URL.
    :param mlflow_url:
    :return: A tuple with the device flow url and the mlflow instance's client_id
    """
    keycloak_query = requests.get(f"{mlflow_url}/oauth2/start", allow_redirects=False)
    keycloak_url = urllib.parse.urlparse(keycloak_query.next.url)
    client_id = urllib.parse.parse_qs(keycloak_url.query)['client_id'][0]

    # Blank out the query args
    parts = list(keycloak_url[0:6])
    parts[4] = ''
    device_flow_start_url = urllib.parse.urlunparse(parts)
    return device_flow_start_url, client_id


def run():
    realm_url, client_id = get_keycloak_realm_and_client(os.environ['MLFLOW_TRACKING_URI'])

    r = requests.post(realm_url+"/device", data=[("client_id", client_id)])
    device_code = r.json()['device_code']
    print(f"Visit this link to authorize access to MLFlow instance {client_id}")
    print(r.json()['verification_uri_complete'])

    token_query_url = realm_url.replace("protocol/openid-connect/auth",
                                        "protocol/openid-connect/token")

    token = None
    while not token:
        time.sleep(5)
        r2 = requests.post(
            token_query_url,
            data=[
                ("client_id", client_id),
                ("grant_type", "urn:ietf:params:oauth:grant-type:device_code"),
                ("device_code", device_code)
            ])
        if r2.status_code==200:
            token = r2.json()['access_token']

    os.environ["MLFLOW_TRACKING_TOKEN"] = token

    if "--echo" in sys.argv:
        print(f"export MLFLOW_TRACKING_TOKEN={token}")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mlflow_token']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.2,<3.0.0']

entry_points = \
{'console_scripts': ['mlflow-token = mlflow_token.mlflow_token:run']}

setup_kwargs = {
    'name': 'mlflow-token',
    'version': '0.1.2a1',
    'description': 'Command line tool to retreive access token for mlflow instance',
    'long_description': "# mlflow-token\nObtain an access token for an MLFlow instance deployed behind OAuth2-proxy and\nkeycloak. \n\nThis script will use your current setting of `MLFLOW_TRACKING_URI` to look for\nthe keycloak redirect from it's OAuth2-proxy. From there it will start an\nOAuth device flow to allow you to obtain a valid access token. This token will\nbe set in your environment's `MLFLOW_TRACKING_TOKEN` variable for use by the\nMLFlow client libraries.\n\nYou can also specify the `--echo` command line argument to get the tool to print\na command for assigning the token to the environment variable.\n",
    'author': 'Ben Galewsky',
    'author_email': 'bengal1@illinois.edu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ncsa/mlflow-token',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
